--do not load the addon if current player is not a shaman
a, b, classId = UnitClass("player")
if (classId ~= 7) then return end

local LastSpellCastSucceededTimes = { 0, 0, 0, 0}
local ActiveTotems = 
{
	[1] = { SpellId = 0, StartTime = 0 },
	[2] = { SpellId = 0, StartTime = 0 },
	[3] = { SpellId = 0, StartTime = 0 },
	[4] = { SpellId = 0, StartTime = 0 }
}


local function ClearTotem(totemIndex)
	ActiveTotems[totemIndex].SpellId = 0
	ActiveTotems[totemIndex].StartTime = 0
	
	UIElements[totemIndex].Image:SetTexture(nil)
	UpdateTotemData()
end

function PlayerTotemUpdate(totemSlot)
	--in Blizzard's api earth and fire indexes are switched. switch them back here
	totemIndex = totemSlot
	if (totemSlot == 1) then totemIndex = 2 end
	if (totemSlot == 2) then totemIndex = 1 end
	
	--if the last succeeded time for this totem type is less than 0.75 sec ago from now ignore this event. this is necessary because sometimes PLAYER_TOTEM_UPDATE gets called twice for the same totem event for no apparent reason
	lastSucceeded = LastSpellCastSucceededTimes[totemIndex]
	if (GetTime() < (lastSucceeded + 0.75)) then return end
	
	--if a totem currently exists for this slot then it must have been destroyed. clear data
	if (ActiveTotems[totemIndex].SpellId ~= 0) then ClearTotem(totemIndex) end
	
	UpdateTotemData()
end

function SpellCastSucceeded(spellId)
	totemIndex = GetSpellTotemIndex(spellId)
	if (totemIndex == nil) then return end
	
	LastSpellCastSucceededTimes[totemIndex] = GetTime()
	
	--update active totems and totem data
	ActiveTotems[totemIndex].SpellId = spellId
	ActiveTotems[totemIndex].StartTime = GetTime()
	UpdateTotemData()
end

function UpdateTotemData()
	for totemIndex = 1, 4 
	do
		spellId = ActiveTotems[totemIndex].SpellId
		local icon = GetSpellTexture(spellId)
		UIElements[totemIndex].Image:SetTexture(icon)
		
		RefreshTimer(totemIndex)
		RefreshAffectedCount(totemIndex)
	end
end

--[[
	Get the number of players affected by the totem with the given index.
]]
local function GetAffectedCount(totemIndex)
	local unitsAffected = 0
	local allUsersAffected = true

	--get the number of units = 1 (player) + party members
	unitCount = 1 + math.min(4, GetNumGroupMembers())
	for unitIndex = 1, unitCount 
	do
		--get ID of this unit and the totem aura (spell) ID we are looking for
		unitId = GetUnitId(unitIndex)
		totemAuraId = Totems[spellId].AuraId;
		
		if (IsUnitAffected(unitId, totemAuraId)) 
		then 
			--if unit is affected by this totem, increment affected if yes
			unitsAffected = unitsAffected + 1 
		else
			--check if totem is useful for this class. if yes mark not all users as affected
			isUseful = IsAuraUsefulToUnit(unitId, totemAuraId)
			if (isUseful) then allUsersAffected = false end
		end
	end
	
	return unitsAffected, allUsersAffected
end

--[[
	Updates totem timers and affected count at a certain interval.
]]
local _timeSinceLastUpdate = 0
local function OnUpdate(self, elapsed)
	--update time since last update and return if enough time hasn't elapsed
    _timeSinceLastUpdate = _timeSinceLastUpdate + elapsed
    if (_timeSinceLastUpdate < 1) then return end

	UpdateTotemData()
	_timeSinceLastUpdate = 0
end

--[[
	Refreshes the number of players affected by the totem with the given index.
]]
function RefreshAffectedCount(totemIndex)
	local spellId = ActiveTotems[totemIndex].SpellId
	totemCountText = UIElements[totemIndex].Count
	
	--if no totem found set empty text and return
	if (spellId == 0)
	then
		totemCountText:SetText("");
		return
	end
	
	--if affected count should be ignored for this totem set white text and —
	if (Totems[spellId].AuraId == 0)
	then
		totemCountText:SetText("—");
		totemCountText:SetTextColor(1.0, 1.0, 1.0, 1.0)
	else
		local unitsAffected, allUsersAffected = GetAffectedCount(totemIndex)
		totemCountText:SetText(unitsAffected);
		
		if (allUsersAffected) 
		then 
			totemCountText:SetTextColor(0.0, 1.0, 0.0, 1.0)
		else
			totemCountText:SetTextColor(1.0, 0.0, 0.0, 1.0)
		end
	end
end

--[[
	Refreshes the time left until expiry of the totem with the given index.
]]
function RefreshTimer(totemIndex)
	local timeLeftText = ""
	
	startTime = ActiveTotems[totemIndex].StartTime
	
	if (startTime ~= 0) 
	then
		--get duration for this totem
		spellId = ActiveTotems[totemIndex].SpellId
		duration = Totems[spellId].Duration
		
		timeElapsed = GetTime() - startTime
		timeLeft = duration - timeElapsed
		if (timeLeft < 0) then timeLeft = 0 end
		
		minutes = string.format("%2.f", math.floor(timeLeft / 60))
		seconds = string.format("%02.f", math.floor(timeLeft  - minutes * 60))
		timeLeftText = minutes .. ":" .. seconds
	end
	
	UIElements[totemIndex].Timer:SetText(timeLeftText)
end

--[[
	Zone changed implies all totems destroyed. 
	For some reason PLAYER_TOTEM_UPDATE in that case so this is a workaround.
]]
function ZoneChanged()
	for totemIndex = 1, 4 
	do
		ClearTotem(totemIndex)
	end
end

local function CreateTextFrame(parentFrame, x, y, frameAnchor, textAnchor, textSize)
	local frame = CreateFrame("FRAME", nil, parentFrame)
	frame:SetPoint(frameAnchor, parentFrame, x, y)
	frame:SetWidth(39.375)
	frame:SetHeight(20)
		
	local fontString = frame:CreateFontString()
	fontString:SetPoint(textAnchor)
	fontString:SetFont("Fonts\\FRIZQT__.TTF", textSize, "OUTLINE")
	
	return fontString
end

local function CreateSubFrame(parentFrame, x)
	local frame = CreateFrame("Button", nil, parentFrame, "ActionButtonTemplate")
	frame:SetPoint("LEFT", parentFrame, "LEFT", x, 2) 

	local texture = frame:CreateTexture(nil, "OVERLAY")
	texture:SetAllPoints(frame)

	frame.texture = texture
	frame:SetFrameStrata("BACKGROUND")
	frame:SetWidth(36)
	frame:SetHeight(36)
	
	return texture
end

--create the main frame
mainFrame = CreateFrame("FRAME", nil, UIParent, "TooltipBorderedFrameTemplate")
mainFrame:SetFrameStrata("BACKGROUND")
mainFrame:SetWidth(175)
mainFrame:SetHeight(85)

local function Image(x)
	return CreateSubFrame(mainFrame, x)
end

local function Timer(x)
	return CreateTextFrame(mainFrame, x, -3, "TOPLEFT", "LEFT", 11);
end

local function Count(x)
	return CreateTextFrame(mainFrame, x, 5, "BOTTOMLEFT", "CENTER", 14);
end

UIElements = 
{ 
	[1] = { Image = Image(10), Timer = Timer(10), Count = Count(10) }, 
	[2] = { Image = Image(50), Timer = Timer(50), Count = Count(50) }, 
	[3] = { Image = Image(90), Timer = Timer(90), Count = Count(90) }, 
	[4] = { Image = Image(130), Timer = Timer(130), Count = Count(130) } 
}

function SaveMainFrameLocation()
	a, b, c, x, y = mainFrame:GetPoint()
	MainFrameX = x
	MainFrameY = y

	mainFrame:StopMovingOrSizing()
end

--set the vars/events necessary to be able to move the frame and save its location
mainFrame:SetMovable(true)
mainFrame:EnableMouse(true)
mainFrame:SetUserPlaced(true);
mainFrame:RegisterForDrag("LeftButton")
mainFrame:SetScript("OnDragStart", mainFrame.StartMoving)
mainFrame:SetScript("OnDragStop", SaveMainFrameLocation)

local function AddonLoaded()
	RestoreMainFrameLocation()
	UpdateTotemData()
end

function mainFrame:OnEvent(eventName, arg1, arg2, arg3)
	if eventName == "ADDON_LOADED" then AddonLoaded() end
	if eventName == "ZONE_CHANGED_NEW_AREA" then ZoneChanged() end
	if eventName == "PLAYER_TOTEM_UPDATE" then PlayerTotemUpdate(arg1) end
	if eventName == "UNIT_SPELLCAST_SUCCEEDED" then SpellCastSucceeded(arg3) end
end

--register events and handler
mainFrame:RegisterEvent("ADDON_LOADED")
mainFrame:RegisterEvent("PLAYER_TOTEM_UPDATE")
mainFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
mainFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
mainFrame:SetScript("OnUpdate", OnUpdate) 
mainFrame:SetScript("OnEvent", mainFrame.OnEvent);

function RestoreMainFrameLocation()
	--set defaults location if the addon has not been loaded before
	if MainFrameX == nil or MainFrameY == nil then
		mainFrame:SetPoint("CENTER", 0, 0)
		return
	end

	--set frame location
	mainFrame:SetPoint("TOPLEFT", MainFrameX, MainFrameY)
end

mainFrame.texture = mainFrame:CreateTexture(nil, "LOW")
mainFrame.texture:SetPoint("BOTTOMLEFT", mainFrame, "BOTTOMLEFT", 3, 3);
mainFrame.texture:SetPoint("BOTTOMRIGHT", mainFrame, "BOTTOMRIGHT", 3, 3);
mainFrame.texture:SetPoint("TOPLEFT", mainFrame, "TOPLEFT", 3, -3);
mainFrame.texture:SetPoint("TOPRIGHT", mainFrame, "TOPRIGHT", -3, -3);
mainFrame.texture:SetColorTexture(0.0, 0.0, 0.0, 0.5)
  
mainFrame:Show()